@@ -36,6 +36,11 @@ Static.ident {
   bg: "blue";
 }
 
+Static.transient {
+  fg: "yellow";
+  b: 1;
+}
+
 EOF
 
 0x55AA;
